package com.thomasbet.endpoints.controller;

import com.thomasbet.endpoints.DTO.UserDTO;
import com.thomasbet.endpoints.Model.User;
import com.thomasbet.endpoints.service.UserService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.security.crypto.bcrypt.BCryptPasswordEncoder;
import org.springframework.web.bind.annotation.*;

import java.util.List;
import java.util.Optional;


@RestController
@RequestMapping("/api/users")
public class UserRestAPI {

    @Autowired
    BCryptPasswordEncoder bCryptPasswordEncoder;

    @Autowired
    private UserService userService;

    //vrácení všech uživatelů
    @GetMapping
    public ResponseEntity<List<User>> getAllUsers(@RequestParam(required = false) String email,
              @RequestParam(required = false) String nickname){
        List<User> users = userService.getAllUsers();

        if (email != null) {
            users = users.stream().filter(user -> user.getEmail().contains(email)).toList();
        } else if (nickname != null) {
            users = users.stream().filter(user -> user.getNickname().contains(nickname)).toList();
        }

        if(!users.isEmpty()){
            return ResponseEntity.ok(users);
        }else{
            return ResponseEntity.notFound().build();
        }
    }

    //vrácení uživatele podle ID
    @GetMapping("/{id}")
    public ResponseEntity<User> getUserById(@PathVariable Long id) {
        User user = userService.getUserById(id);
        if (user != null) {
            return ResponseEntity.ok(user);
        } else {
            return ResponseEntity.notFound().build();
        }
    }


    //vytvoření uživatele při registraci
    @PostMapping
    public ResponseEntity<User> createUser(@RequestBody User user) {
        User newUser = userService.createUser(user);
        return ResponseEntity.status(HttpStatus.CREATED).body(newUser);
    }

    //změna dat u uživatele (role)
    @PatchMapping
    public ResponseEntity<User> updateUser(@RequestBody User user,
                                           @RequestParam(required = false) Integer deposit) {
        User updatedUser;

        if(deposit == null){
            updatedUser = userService.updateUser(user);
        }else{
            updatedUser = userService.getUserById(user.getId());
            updatedUser.setBalance(user.getBalance() + deposit);
            updatedUser = userService.updateUser(updatedUser);
        }

        if (updatedUser != null) {
            return ResponseEntity.ok(updatedUser);
        } else {
            return ResponseEntity.notFound().build();
        }
    }


    //přihlášení uživatele, vrácení uživatele po přihlášení
    @GetMapping("/login")
    public ResponseEntity<User> getUserAfterLogin(@RequestParam String email){
        Optional<User> user = userService.findUserByEmail(email);

        return user.map(ResponseEntity::ok).orElseGet(() -> ResponseEntity.notFound().build());
    }


    //registrace uživatele, vrácení uživatele po registraci
    @PostMapping("/register")
    public ResponseEntity<User> getUserAfterRegister(@RequestBody User user){
        Optional<User> optionalUser = userService.findUserByEmail(user.getEmail());
        if(optionalUser.isEmpty()){
            return ResponseEntity.ok(userService.createUser(user));
        }else{
            return ResponseEntity.notFound().build();
        }
    }

    //smazání uživatele NEPOUŽITO
    @DeleteMapping("/{id}")
    public ResponseEntity<Void> deleteUser(@PathVariable Long id) {
        userService.deleteUser(id);
        return ResponseEntity.noContent().build();
    }

}
