package com.thomasbet.endpoints.service;

import com.thomasbet.endpoints.DTO.BlogDTO;
import com.thomasbet.endpoints.DTO.EventDTO;
import com.thomasbet.endpoints.Model.Blog;
import com.thomasbet.endpoints.Model.Event;
import com.thomasbet.endpoints.Model.User;
import com.thomasbet.endpoints.mappers.EventMapper;
import com.thomasbet.endpoints.repository.EventRepository;
import com.thomasbet.endpoints.repository.SportRepository;
import com.thomasbet.endpoints.time_constants.TimeConstants;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Example;
import org.springframework.data.domain.ExampleMatcher;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.List;
import java.util.Optional;

@Service
public class EventService {

    @Autowired
    private EventRepository eventRepository;

    @Autowired
    private SportService sportService;

    @Autowired
    private ScoreService scoreService;

    @Autowired
    private EventMapper eventMapper;

    //servis pro eventy(zápasy)
    public EventService(EventRepository eventRepository, SportService sportService, ScoreService scoreService, EventMapper eventMapper) {
        this.eventRepository = eventRepository;
        this.sportService = sportService;
        this.scoreService = scoreService;
        this.eventMapper = eventMapper;
    }

    public List<EventDTO> getAllEvents(){
        return eventRepository.findAll().stream()
                .map(event -> eventMapper.eventToEventDTO(event, scoreService)).toList();
    }

    public EventDTO findById(String eventId){
        Optional<EventDTO> optionalEvent = eventRepository.findById(eventId).map(event -> eventMapper.eventToEventDTO(event, scoreService));
        return optionalEvent.orElse(null);
    }

    public EventDTO saveEvent(EventDTO eventDTO){
        return eventMapper.eventToEventDTO(eventRepository.save(eventMapper.eventDTOToEvent(eventDTO)), scoreService);
    }

    public Page<EventDTO> getEventsWithPagination(int offset, int page, String sportType, boolean upcoming){
        long currentDate = System.currentTimeMillis();
        String filterValue;

        switch(sportType) {
            case "football":
                filterValue = "Soccer";
                break;
            case "ice_hockey":
                filterValue = "Ice Hockey";
                break;
            default:
                if(upcoming){
                    return eventRepository.findByCommenceTimeGreaterThan(currentDate + TimeConstants.ONE_HOUR(), PageRequest.of(page, offset))
                            .map(entity -> eventMapper.eventToEventDTO(entity, scoreService));
                }else{
                    return eventRepository.findByCommenceTimeLessThan(currentDate - (TimeConstants.ONE_HOUR()*3), PageRequest.of(page, offset))
                            .map(entity -> eventMapper.eventToEventDTO(entity, scoreService));
                }
        }

        if (upcoming) {
            return eventRepository.findByCommenceTimeGreaterThanAndSportType(currentDate + TimeConstants.ONE_HOUR(), filterValue, PageRequest.of(page, offset))
                    .map(entity -> eventMapper.eventToEventDTO(entity, scoreService));
        }else{
            return eventRepository.findByCommenceTimeLessThanAndSportType(currentDate - (TimeConstants.ONE_HOUR()*3), filterValue, PageRequest.of(page, offset))
                    .map(entity -> eventMapper.eventToEventDTO(entity, scoreService));
        }
    }


    //převádění na českou lokalizaci času, vracení eventů, převod Socceru na fotbal
    public int getEventCount(int offset, String sportType, boolean upcoming){
        List<Event> events = new ArrayList<>();
        if(sportType.equals("all")){
            if(upcoming){
                events = eventRepository.findAll().stream()
                        .filter(event -> (event.getCommenceTime() + TimeConstants.ONE_HOUR()) > System.currentTimeMillis()).toList();
            }else{
                events = eventRepository.findAll().stream()
                        .filter(event -> (event.getCommenceTime() - 3*TimeConstants.ONE_HOUR()) < System.currentTimeMillis()).toList();
            }

        }else if(sportType.equals("football")){
            if(upcoming){
                events = eventRepository.findAll().stream()
                        .filter(event -> event.getSportType().equals("Soccer"))
                        .filter(event -> (event.getCommenceTime() + TimeConstants.ONE_HOUR()) > System.currentTimeMillis())
                        .toList();
            }else{
                events = eventRepository.findAll().stream()
                        .filter(event -> event.getSportType().equals("Soccer"))
                        .filter(event -> (event.getCommenceTime() - 3*TimeConstants.ONE_HOUR()) < System.currentTimeMillis())
                        .toList();
            }

        }else if(sportType.equals("ice_hockey")){
            if(upcoming){
                events = eventRepository.findAll().stream()
                        .filter(event -> event.getSportType().equals("Ice Hockey"))
                        .filter(event -> (event.getCommenceTime() + TimeConstants.ONE_HOUR()) > System.currentTimeMillis())
                        .toList();
            }else{
                events = eventRepository.findAll().stream()
                        .filter(event -> event.getSportType().equals("Ice Hockey"))
                        .filter(event -> (event.getCommenceTime() - 3*TimeConstants.ONE_HOUR()) < System.currentTimeMillis())
                        .toList();
            }
        }
        return (int) Math.ceil((double) events.size()/offset);
    }


    @Transactional
    public void removeAllData(){
        eventRepository.deleteAll();
    }
}
